@@ -7,15 +7,10 @@ impl<T, N> Serialize for GenericArray<T, N>
           N: ArrayLength<T>
 {
     #[inline]
-    fn serialize<S>(&self, serializer: &mut S) -> Result<(), S::Error>
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
         where S: Serializer
     {
-        // serializes this array just like a slice or a vector
-        let mut state = try!(serializer.serialize_seq(Some(N::to_usize())));
-        for e in self.iter() {
-            try!(serializer.serialize_seq_elt(&mut state, e));
-        }
-        serializer.serialize_seq_end(state)
+        serializer.collect_seq(self.iter())
     }
 }
 
@@ -23,7 +18,7 @@ impl<T, N> Deserialize for GenericArray<T, N>
     where T: Deserialize + Clone,
           N: ArrayLength<T>
 {
-    fn deserialize<D>(deserializer: &mut D) -> Result<GenericArray<T, N>, D::Error>
+    fn deserialize<D>(deserializer: D) -> Result<GenericArray<T, N>, D::Error>
         where D: Deserializer
     {
         // this implementation has the cost of allocating a new vector each time.
